"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const nanoevents_1 = __importDefault(require("nanoevents"));
const optimisticUi_1 = require("./optimisticUi");
const sync_1 = require("./sync");
const globalWindow = typeof window === "undefined" ? {} : window;
function create(params) {
    const { initialData, network } = params;
    const onError = params.onError ||
        (err => {
            throw err;
        });
    const window = params.window || globalWindow;
    const debugConfig = {
        failingWrites: false,
        pretendOffline: false,
        optimisticUIEnabled: true
    };
    const canonData = initialData;
    const events = new nanoevents_1.default();
    const { connectivity, observe, push } = sync_1.sync({
        canonData,
        network,
        onChange: (kind, id) => events.emit("change", [kind, id]),
        onConnectivityChange: () => events.emit("connectivity-changed", undefined),
        onError,
        shouldCrashWrites: () => debugConfig.failingWrites
    });
    const { pendingTransactionCount, update, withPendingTransactions } = optimisticUi_1.optimisticUi({
        commitTransaction: push,
        onChangePendingTransactionCount: () => events.emit("pending-transaction-count-changed", undefined),
        onError,
        onRerenderKind: (kind, id) => events.emit("change", [kind, id]),
        optimisticUIEnabled: () => debugConfig.optimisticUIEnabled
    });
    window.addEventListener("beforeunload", function (e) {
        // We can't ever save lost data anymore
        if (connectivity() === "crashed")
            return;
        if (pendingTransactionCount() === 0)
            return;
        e.preventDefault();
        e.returnValue = "";
    });
    const db = {
        _internals: {
            canonData,
            debugConfig,
            events,
            pendingTransactionCount,
            withPendingTransactions
        },
        connectivity: () => debugConfig.pretendOffline ? "offline" : connectivity(),
        observe,
        update
    };
    return db;
}
exports.create = create;
//# sourceMappingURL=create.js.map