"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const push_1 = require("./push");
function makeCounters() {
    const stringify = (a, b) => `${a}-${b}`;
    const data = {};
    return {
        add: function (a, b, f) {
            const kind = stringify(a, b);
            // Cancellation is still not in the protocol,
            // so this is stub logic for now that never cancels.
            if (!data[kind])
                data[kind] = { count: 1, cancel: f() };
            return () => { };
            // if (!data[kind] || data[kind].count === 0) {
            //   data[kind] = { count: 0, cancel: f() };
            // }
            // data[kind].count += 1;
            // return function() {
            //   data[kind].count -= 1;
            //   if (data[kind].count === 0) {
            //     data[kind].cancel();
            //     data[kind].cancel = () => {
            //       throw new Error(`Assertion error`);
            //     };
            //   }
            // };
        }
    };
}
function sync(params) {
    const { canonData, network, onChange, onConnectivityChange, onError, shouldCrashWrites } = params;
    let connectivity = "connecting";
    function setConnectivity(value) {
        if (connectivity === "crashed")
            return;
        connectivity = value;
        onConnectivityChange();
    }
    const pushesInFlight = {};
    const net = network({
        onChange: function ({ kind, id, revision, value }) {
            canonData[kind][id] = { revision, value };
            onChange(kind, id);
        },
        onConnectivityChange: setConnectivity,
        onError,
        onPushResult: (pushId, result) => {
            if (!pushesInFlight[pushId])
                return;
            pushesInFlight[pushId](result);
            delete pushesInFlight[pushId];
        }
    });
    const push = push_1.makePush({
        canonData,
        onError: err => {
            setConnectivity("crashed");
            throw err;
        },
        onNetPush: params => new Promise(resolve => {
            pushesInFlight[params.pushId] = resolve;
            net.push(params);
        }),
        onUpdate: ({ kind, id, newValue, newRevision }) => {
            canonData[kind][id] = {
                revision: newRevision,
                value: newValue
            };
        },
        shouldCrashWrites
    });
    const counters = makeCounters();
    return {
        connectivity: () => connectivity,
        observe: (kind, id) => {
            return counters.add(kind, id, function () {
                return net.getAndObserve(kind, id);
            });
        },
        push
    };
}
exports.sync = sync;
//# sourceMappingURL=index.js.map