"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function makeThrottled(f) {
    const scheduleStore = {};
    function scheduleFor(kind, id) {
        scheduleStore[kind] = scheduleStore[kind] || {};
        scheduleStore[kind][id] = scheduleStore[kind][id] || {};
        return scheduleStore[kind][id];
    }
    return function (kind, id) {
        const schedule = scheduleFor(kind, id);
        function run() {
            if (schedule.debounceTimer)
                clearTimeout(schedule.debounceTimer);
            if (schedule.maxTimer)
                clearTimeout(schedule.maxTimer);
            schedule.maxTimer = null;
            f(kind, id);
        }
        if (schedule.debounceTimer)
            clearTimeout(schedule.debounceTimer);
        schedule.debounceTimer = setTimeout(run, 500);
        if (!schedule.maxTimer) {
            schedule.maxTimer = setTimeout(run, 10000);
        }
    };
}
exports.makeThrottled = makeThrottled;
//# sourceMappingURL=throttled.js.map