"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const defaultOnMissing = (kind, id) => {
    throw new Error(`Missing dummy data ${kind} ${id}`);
};
function makeDummyAdapter(data, opts = {}) {
    // Clone data one level deep
    const _ = data;
    data = {};
    Object.keys(_).forEach(k => {
        data[k] = Object.assign({}, _[k]);
    });
    const onMissing = opts.onMissing || defaultOnMissing;
    let locked = false;
    return function ({ onChange, onConnectivityChange, onPushResult }) {
        setTimeout(() => onConnectivityChange("online"), 1000);
        return {
            getAndObserve: function (kind, id) {
                setTimeout(function () {
                    onChange({
                        kind,
                        id,
                        revision: "dummy",
                        value: id in data[kind] ? data[kind][id] : onMissing(kind, id)
                    });
                }, 1500);
                return () => { };
            },
            push: ({ kind, id, pushId, value }) => {
                if (locked) {
                    setTimeout(() => onPushResult(pushId, "conflict"), 500);
                    return;
                }
                locked = true;
                setTimeout(() => {
                    data[kind][id] = value;
                    onChange({ kind, id, revision: "dummy", value });
                    onPushResult(pushId, { newRevision: "dummy", newValue: value });
                    locked = false;
                }, 2000);
            }
        };
    };
}
exports.makeDummyAdapter = makeDummyAdapter;
//# sourceMappingURL=dummy.js.map