"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const reconnecting_websocket_1 = __importDefault(require("reconnecting-websocket"));
const serialization_1 = require("./serialization");
__export(require("./memoryServer"));
var server_1 = require("./server");
exports.runWebsocketServer = server_1.runWebsocketServer;
function makeWsProtocolAdapter(params) {
    const { url, _socket } = params;
    const serialization = params.serialization || serialization_1.jsonSerialization;
    const socket = _socket || new reconnecting_websocket_1.default(url);
    let pingTimer;
    let timeoutTimer;
    function restartPingMachine() {
        clearTimeout(pingTimer);
        clearTimeout(timeoutTimer);
        pingTimer = setTimeout(function () {
            send({ action: "ping" });
            timeoutTimer = setTimeout(() => socket.reconnect(), 15000);
        }, 15000);
    }
    restartPingMachine();
    const send = (input) => {
        socket.send(JSON.stringify(input));
    };
    const messagesOnEveryReconnect = [];
    return {
        adapter: function ({ onChange, onConnectivityChange, onError, onPushResult }) {
            socket.onopen = () => {
                onConnectivityChange("online");
                messagesOnEveryReconnect.forEach(send);
            };
            socket.onclose = () => onConnectivityChange("offline");
            socket.onerror = () => onError(new Error("WebSocket error. We can’t retrieve details about the error because the browser does not provide them for security reasons."));
            const handlers = {
                clientError: (msg) => {
                    onError(new Error(msg.message));
                },
                pong: () => { },
                pushResult: (msg) => {
                    if (msg.result === "success")
                        onPushResult(msg.pushId, {
                            newRevision: msg.newRevision,
                            newValue: serialization.decode(msg.newValue)
                        });
                    else
                        onPushResult(msg.pushId, msg.result);
                },
                update: (msg) => onChange({
                    kind: msg.kind,
                    id: msg.id,
                    revision: msg.revision,
                    value: serialization.decode(msg.value)
                })
            };
            socket.onmessage = function (event) {
                restartPingMachine();
                const msg = JSON.parse(event.data);
                const action = msg.action;
                if (!action)
                    return console.warn(`Unrecognized message`, msg);
                const handler = handlers[action];
                if (!handler)
                    return console.warn(`Unrecognized message`, msg);
                handler(msg);
            };
            return {
                getAndObserve: (kind, id) => {
                    const msg = { action: "subscribe", kind, id };
                    if (socket.readyState === 1)
                        send(msg);
                    messagesOnEveryReconnect.push(msg);
                    return () => { };
                },
                push: ({ kind, id, pushId, lastSeenRevision, value }) => {
                    send({
                        action: "push",
                        kind,
                        id,
                        pushId,
                        lastSeenRevision,
                        value: serialization.encode(value)
                    });
                }
            };
        },
        auth: token => {
            const msg = { action: "auth", token };
            const currentIndex = messagesOnEveryReconnect.findIndex(m => m.action === "auth");
            if (currentIndex === -1)
                messagesOnEveryReconnect.push(msg);
            else
                messagesOnEveryReconnect[currentIndex] = msg;
            if (socket.readyState === 1)
                send(msg);
        }
    };
}
exports.makeWsProtocolAdapter = makeWsProtocolAdapter;
//# sourceMappingURL=index.js.map