"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const server_1 = require("./server");
const wait = (ms) => new Promise(resolve => setTimeout(resolve, ms));
function runMemoryServer(params) {
    const { initialData, port, serialization, server } = params;
    const data = {};
    Object.keys(initialData).forEach(kind => {
        data[kind] = {};
        Object.keys(initialData[kind]).forEach(id => {
            data[kind][id] = { revision: "1", value: initialData[kind][id] };
        });
    });
    function getItem(kind, id) {
        const item = data[kind][id];
        if (!item)
            throw new Error(`Invalid item ${kind} ${id}`);
        return item;
    }
    server_1.runWebsocketServer({
        onChangeData: function ({ kind, id, value }) {
            return __awaiter(this, void 0, void 0, function* () {
                yield wait(500);
                const curr = getItem(kind, id);
                curr.revision = String(Number(curr.revision) + 1);
                curr.value = value;
                return { newRevision: curr.revision, newValue: curr.value };
            });
        },
        onRequestData: function ({ kind, id, send }) {
            send(getItem(kind, id));
        },
        port: port,
        serialization,
        server: server
    });
}
exports.runMemoryServer = runMemoryServer;
//# sourceMappingURL=memoryServer.js.map