"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function makeWsMock() {
    let connectionListener;
    const server = {
        on: function (event, cb) {
            if (event === "connection") {
                if (connectionListener)
                    throw new Error("Not implemented");
                connectionListener = cb;
                return server;
            }
            throw new Error("Not implemented");
        }
    };
    return {
        client: (onServerMessage) => {
            let onClientMessage;
            const socket = {
                on: function (event, cb) {
                    if (event === "message") {
                        if (onClientMessage)
                            throw new Error("Not implemented");
                        onClientMessage = cb;
                        return;
                    }
                    throw new Error("Not implemented");
                },
                readyState: 1,
                send: (data) => onServerMessage(JSON.parse(data))
            };
            connectionListener(socket);
            return {
                send: (msg) => onClientMessage(Buffer.from(JSON.stringify(msg)))
            };
        },
        server
    };
}
exports.makeWsMock = makeWsMock;
//# sourceMappingURL=ws-mock.js.map