"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function makeUpdater(db, kind, id) {
    const updater = function (_options, delta) {
        db.update(kind, id, delta);
    };
    return (a, b) => updater(b ? a : {}, b || a);
}
function makeHooks(params) {
    const { React, useTryDB } = params;
    function useRerender() {
        const [, rerender] = React.useReducer(x => x + 1, 0);
        return () => {
            rerender({});
        };
    }
    return {
        useConnectivity: () => {
            const db = useTryDB();
            if (db === "missing-provider")
                return "online";
            const rerender = useRerender();
            React.useEffect(() => db._internals.events.on("connectivity-changed", rerender), []);
            return db.connectivity();
        },
        useData: function (kind, id) {
            const db = useTryDB();
            if (db === "missing-provider")
                throw new Error("The component you render needs a DBProvider grand-parent");
            const rerender = useRerender();
            React.useEffect(() => db._internals.events.on("change", function (k) {
                if (k[0] === kind && k[1] === id)
                    rerender();
            }), []);
            React.useEffect(() => db.observe(kind, id), [kind, id]);
            const t = db._internals.withPendingTransactions(db._internals.canonData)[kind][id];
            return [
                t ? t.value : "loading",
                React.useCallback(makeUpdater(db, kind, id), [kind, id])
            ];
        },
        useOncillaDebug: () => {
            const db = useTryDB();
            if (db === "missing-provider")
                throw new Error("The component you render needs a DBProvider grand-parent");
            const { debugConfig, events } = db._internals;
            const rerender = useRerender();
            React.useEffect(() => events.on("debug-config-changed", rerender), []);
            return [
                debugConfig,
                React.useCallback(function (updates) {
                    Object.assign(debugConfig, updates);
                    events.emit("debug-config-changed", undefined);
                }, [])
            ];
        },
        usePendingChanges: () => {
            const db = useTryDB();
            if (db === "missing-provider")
                return false;
            const rerender = useRerender();
            React.useEffect(() => db._internals.events.on("pending-transaction-count-changed", rerender), []);
            return db._internals.pendingTransactionCount();
        }
    };
}
exports.makeHooks = makeHooks;
//# sourceMappingURL=hooks.js.map