"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function makeSemiControlledInput(React) {
    return function SemiControlledInput(props) {
        const { defaultValue, onBlur, onChange, onFocus, value } = props;
        if (defaultValue !== undefined)
            throw new Error(`Use SemiControlledInput with onChange and value, not defaultValue`);
        const [focused, setFocused] = React.useState(false);
        const [valueUnderEdit, setValueUnderEdit] = React.useState(value);
        return React.createElement("input", Object.assign({}, props, { onBlur: e => {
                if (onChange && valueUnderEdit !== value)
                    onChange({ target: e.target });
                if (onBlur)
                    onBlur(e);
                setFocused(false);
            }, onChange: e => setValueUnderEdit(e.target.value), onFocus: e => {
                setValueUnderEdit(value);
                setFocused(true);
                if (onFocus)
                    onFocus(e);
            }, value: focused ? valueUnderEdit : value }));
    };
}
exports.makeSemiControlledInput = makeSemiControlledInput;
//# sourceMappingURL=inputs.js.map